# import the necessary packages
from picamera.array import PiRGBArray
from picamera import PiCamera
import datetime
import imutils
import time
import cv2

# Define constants
min_upload_seconds = 3.0
min_motion_frames = 6
camera_warmup_time = 2.5
delta_thresh = 5
resolution = [1280, 860]
fps = 8
min_area = 3000

# initialize the camera and grab a reference to the raw camera capture
camera = PiCamera()
camera.resolution = tuple(resolution)
camera.framerate = fps
rawCapture = PiRGBArray(camera, size=tuple(resolution))
 
# allow the camera to warmup, then initialize the average frame, last
# uploaded timestamp, and frame motion counter
print "[INFO] warming up..."
time.sleep(camera_warmup_time)
avg = None
lastUploaded = datetime.datetime.now()
motionCounter = 0

# capture frames from the camera
for f in camera.capture_continuous(rawCapture, format="bgr", use_video_port=True):
	# grab the raw NumPy array representing the image and initialize
	# the timestamp and occupied/unoccupied text
	rawFrame = f.array
	timestamp = datetime.datetime.now()
	motionDetected = False
 
	# resize the frame, convert it to grayscale, and blur it
	frame = imutils.resize(rawFrame, width=500)
	gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
	gray = cv2.GaussianBlur(gray, (21, 21), 0)
 
	# if the average frame is None, initialize it
	if avg is None:
		print "[INFO] starting background model..."
		avg = gray.copy().astype("float")
		rawCapture.truncate(0)
		continue
 
	# accumulate the weighted average between the current frame and
	# previous frames, then compute the difference between the current
	# frame and running average
	cv2.accumulateWeighted(gray, avg, 0.5)
	frameDelta = cv2.absdiff(gray, cv2.convertScaleAbs(avg))
	
	# threshold the delta image, dilate the thresholded image to fill
	# in holes, then find contours on thresholded image
	thresh = cv2.threshold(frameDelta, delta_thresh, 255,
		cv2.THRESH_BINARY)[1]
	thresh = cv2.dilate(thresh, None, iterations=2)
	(cnts, _) = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL,
		cv2.CHAIN_APPROX_SIMPLE)
 
	# loop over the contours
	for c in cnts:
		# if the contour is too small, ignore it
		if cv2.contourArea(c) < min_area:
			continue
 
		# compute the bounding box for the contour, draw it on the frame,
		# and update the text
		# TODO: Need to scale the bounding boxes from the downsized image to the real image
		#(x, y, w, h) = cv2.boundingRect(c)
		#cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
		motionDetected = True
	
	# check to see if the room is occupied
	if motionDetected == True:
		# check to see if enough time has passed between uploads
		if (timestamp - lastUploaded).seconds >= min_upload_seconds:
			# increment the motion counter
			motionCounter += 1
			
			# check to see if the number of frames with consistent motion is
			# high enough
			if motionCounter >= min_motion_frames:
				# draw the text and timestamp on the frame
				saveFrame = imutils.rotate(rawFrame, angle=180)
				ts = timestamp.strftime("%A %d %B %Y %H:%M:%S")
				timeStampFilename = timestamp.strftime("%Y_%m_%d_%H%M%S")
				fontSize = 1.2
				fontThickness = 2
				cv2.putText(saveFrame, ts, (10, saveFrame.shape[0] - 10), cv2.FONT_HERSHEY_SIMPLEX,
					fontSize, (0, 255, 0), fontThickness)
				
				# write the image to file
				filePath = "/var/www/camera/{fileName}.jpg".format(fileName=timeStampFilename)
				cv2.imwrite(filePath, saveFrame)
				
				# update the last uploaded timestamp and reset the motion
				# counter
				lastUploaded = timestamp
				motionCounter = 0
				
				print "[INFO] motion detected"
 
	# otherwise, the room is not occupied
	else:
		motionCounter = 0
 
	# clear the stream in preparation for the next frame
	rawCapture.truncate(0)