#!/usr/bin/python

import smbus
import time

bus = smbus.SMBus(1) #Depends on which I2C bus is in use

DEVICE_ADDRESS_EEPROM = 0x57	#MCP79410 EEPROM I2C address
DEVICE_ADDRESS_SRAM = 0x6F	#MCP79410 SRAM I2C address

############ MCP79410 register locations #######################
#Time registers
DEVICE_SECONDS = 0x00
DEVICE_MINUTES = 0x01
DEVICE_HOURS = 0x02
DEVICE_DAY = 0x03		# 1-7
DEVICE_DATE = 0x04		# 1-31
DEVICE_MONTH = 0x05
DEVICE_YEAR = 0x06
#Control registers
DEVICE_CONTROL = 0x07
DEVICE_CALIBRATION = 0x08
#Time stamp power down
DEVICE_MINUTE_PD = 0x18
DEVICE_HOUR_PD = 0x19
DEVICE_DATE_PD = 0x1A
DEVICE_MONTH_PD = 0x1B
#Time stamp power up
DEVICE_MINUTE_PU = 0x1C
DEVICE_HOUR_PU = 0x1D
DEVICE_DATE_PU = 0x1E
DEVICE_MONTH_PU = 0x1F

# VBAT bit
VBAT_BIT = 4

################### Define some common functions ###################

# Read minutes (or seconds)
def read_minutes(address):
	minutes = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_minutes_ones = (minutes & 0x0F)
	bcd_minutes_tens = (minutes & 0b01110000)
	bcd_minutes_tens = (bcd_minutes_tens >> 4)
	return (bcd_minutes_ones + (10*bcd_minutes_tens))

# Read hours
def read_hours(address):
	hours = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_hours_ones = (hours & 0x0F)
	bcd_hours_tens = (hours & 0x30)
	bcd_hours_tens = (bcd_hours_tens >> 4)
	return (bcd_hours_ones + (10*bcd_hours_tens))

# Read day of month
def read_day(address):
	date = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_date_ones = (date & 0x0F)
	bcd_date_tens = (date & 0x30)
	bcd_date_tens = (bcd_date_tens >> 4)
	return (bcd_date_ones + (10*bcd_date_tens))

# Read current month
def read_month(address):
	month = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_month_ones = (month & 0x0F)
	bcd_month_tens = (month & 0x10)
	bcd_month_tens = (bcd_month_tens >> 4)
	return (bcd_month_ones + (10*bcd_month_tens))


################### Get time stamps ################################

# Read power down time
minutes_pd = read_minutes(DEVICE_MINUTE_PD)
hours_pd = read_hours(DEVICE_HOUR_PD)
date_pd = read_day(DEVICE_DATE_PD)
month_pd = read_month(DEVICE_MONTH_PD)

# Read power up time
minutes_pu = read_minutes(DEVICE_MINUTE_PU)
hours_pu = read_hours(DEVICE_HOUR_PU)
date_pu = read_day(DEVICE_DATE_PU)
month_pu = read_month(DEVICE_MONTH_PU)

# Clear VBAT bit once time stamps have been read
day = bus.read_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DAY)
day = day & (0xEF)
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DAY, day)

################ Display the time #######################################

# Load power down times
lpd = list(time.localtime())
lpd[4] = minutes_pd
lpd[3] = hours_pd
lpd[2] = date_pd
lpd[1] = month_pd

# Load power up times
lpu = list(time.localtime())
lpu[4] = minutes_pu
lpu[3] = hours_pu
lpu[2] = date_pu
lpu[1] = month_pu

# Format the time nicely
power_down = time.struct_time(lpd)
power_up = time.struct_time(lpu)
power_down_string = time.strftime("%d %b %H:%M", power_down)
power_up_string = time.strftime("%d %b %H:%M", power_up)

# Power down time stamp
print "Power Down:"
print(power_down_string)
#print "%d:%d." %(hours_pd, minutes_pd)
#print "On the %d. of the %d. month." %(date_pd, month_pd)

# Power up time stamp
print "\nPower Up:"
print(power_up_string)
#print "%d:%d." %(hours_pu, minutes_pu)
#print "On the %d. of the %d. month." %(date_pu, month_pu)
