import smbus
import time

bus = smbus.SMBus(1) #Depends on which I2C bus is in use

DEVICE_ADDRESS_EEPROM = 0x57	#MCP79410 EEPROM I2C address
DEVICE_ADDRESS_SRAM = 0x6F	#MCP79410 SRAM I2C address

############ MCP79410 register locations #######################
#Time registers
DEVICE_SECONDS = 0x00
DEVICE_MINUTES = 0x01
DEVICE_HOURS = 0x02
DEVICE_DAY = 0x03		# 1-7
DEVICE_DATE = 0x04		# 1-31
DEVICE_MONTH = 0x05
DEVICE_YEAR = 0x06
#Control registers
DEVICE_CONTROL = 0x07
DEVICE_CALIBRATION = 0x08
#Time stamp power down
DEVICE_MINUTE_PD = 0x18
DEVICE_HOUR_PD = 0x19
DEVICE_DATE_PD = 0x1A
DEVICE_MONTH_PD = 0x1B
#Time stamp power up
DEVICE_MINUTE_PU = 0x1C
DEVICE_HOUR_PU = 0x1D
DEVICE_DATE_PU = 0x1E
DEVICE_MONTH_PU = 0x1F

################### Define some common functions ###################

# Read minutes (or seconds)
def read_minutes(address):
	minutes = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_minutes_ones = (minutes & 0x0F)
	bcd_minutes_tens = (minutes & 0b01110000)
	bcd_minutes_tens = (bcd_minutes_tens >> 4)
	return (bcd_minutes_ones + (10*bcd_minutes_tens))

# Read hours
def read_hours(address):
	hours = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_hours_ones = (hours & 0x0F)
	bcd_hours_tens = (hours & 0x30)
	bcd_hours_tens = (bcd_hours_tens >> 4)
	return (bcd_hours_ones + (10*bcd_hours_tens))

# Read day of month
def read_day(address):
	date = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_date_ones = (date & 0x0F)
	bcd_date_tens = (date & 0x30)
	bcd_date_tens = (bcd_date_tens >> 4)
	return (bcd_date_ones + (10*bcd_date_tens))

# Read current month
def read_month(address):
	month = bus.read_byte_data(DEVICE_ADDRESS_SRAM, address)
	bcd_month_ones = (month & 0x0F)
	bcd_month_tens = (month & 0x10)
	bcd_month_tens = (bcd_month_tens >> 4)
	return (bcd_month_ones + (10*bcd_month_tens))

# Create BCD number
def toBCD(number):
	bcd_ones = number % 10
	number = number / 10
	bcd_tens = number % 10
	bcd_number = bcd_ones | (bcd_tens << 4)
	return bcd_number

################### Initial configuration ##########################

# Stop the oscillator and load the current time
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_SECONDS, 0)

system_time = time.localtime()

# Weekday conversion
# system_time.tm_wday is 0 - 6 with 0 = Monday
# RTC used 1 - 7, with 1 = Friday
weekday = system_time.tm_wday + 4
if weekday > 7:
	weekday = weekday - 7

bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_SECONDS, toBCD(system_time.tm_sec))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_MINUTES, toBCD(system_time.tm_min))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_HOURS, toBCD(system_time.tm_hour))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DAY, toBCD(weekday))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DATE, toBCD(system_time.tm_mday))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_MONTH, toBCD(system_time.tm_mon))
bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_YEAR, toBCD(system_time.tm_year))


# Set 24-hour format
hours = bus.read_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_HOURS)
if (hours & 0x40) == 1:
	hours = hours | 0x40
	bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_HOURS, hours)
	print "24-hour format set."

# Enable the external battery
day = bus.read_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DAY)
#Check if the oscillator needs starting
if (day & 0x08) == 0:
	day = day | 0x08
	bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_DAY, day)
	print "Battery enabled."

#Check if the oscillator needs starting
seconds = bus.read_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_SECONDS)
if (seconds & 0x80) == 0:
	# Write the current time
	# Start the osc
	seconds = seconds | 0x80
	bus.write_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_SECONDS, seconds)
	print "Oscillator started."

################# Get the current time ##############################

seconds = read_minutes(DEVICE_SECONDS)
minutes = read_minutes(DEVICE_MINUTES)
hours = read_hours(DEVICE_HOURS)
date = read_day(DEVICE_DATE)
month = read_month(DEVICE_MONTH)

#Read current year
year = bus.read_byte_data(DEVICE_ADDRESS_SRAM, DEVICE_YEAR)
bcd_year_ones = (year & 0x0F)
bcd_year_tens = (year & 0xF0)
bcd_year_tens = (bcd_year_tens >> 4)
year = bcd_year_ones + (10*bcd_year_tens)


################ Display the time #######################################
# RTC time
print "RTC time:"
print "%d hours, %d minutes, and %d seconds." %(hours, minutes, seconds)
print "On the %d. of the %d. month, year %d since start up." %(date, month, year)

# System time
print "Internet time:"
print "%d hours, %d minutes, and %d seconds." %(system_time.tm_hour, system_time.tm_min, system_time.tm_sec)
print "On the %d. of the %d. month, year %d since start up." %(system_time.tm_mday, system_time.tm_mon, system_time.tm_year)

